#!/bin/sh
PID_DAEMON='/opt/var/lib/dibbler/dibbler.pid'
PID_SCRIPT='/opt/var/lib/dibbler/dibbler.run'
RADVD_CONF='/var/radvd/radvdbr0.conf'
TMP_FILE='/opt/var/lib/dibbler/dibbler.tmp'
LOG_FILE='/opt/var/log/dibbler/dibbler.log'

log() {
    echo -e "$(date '+%F %H:%M:%S') $*" >> $LOG_FILE
}

update_stop() {
    killall dibbler-server 2>/dev/null
    sleep 1
    killall -9 dibbler-server 2>/dev/null
}
update_start() {
    killall -0 dibbler-server 2>/dev/null && return
    dibbler-server start
    log "Dibbler started"
}
undo_firewall() {
    [ -s /opt/var/lib/dibbler/undo_firewall.sh ] || return
    sh /opt/var/lib/dibbler/undo_firewall.sh
    echo -n > /opt/var/lib/dibbler/undo_firewall.sh
    log "Firewall cleaned"
}
undo_routing() {
    [ -s /opt/var/lib/dibbler/undo_routing.sh ] || return
    sh /opt/var/lib/dibbler/undo_routing.sh
    echo -n > /opt/var/lib/dibbler/undo_routing.sh
    log "Routing cleaned"
}
update() {
    local RADIP="$(sed -nr 's~^ *prefix (2003:.*) ~\1~p' /var/radvd/radvdbr0.conf 2>/dev/null | sed 's~:0*~:~g')"
    if [ "$RADIP" == "$(cat $TMP_FILE 2>/dev/null)" ]; then
        log "Nothing changed"
        [ -n "$RADIP" ] && update_start
        return
    fi
    echo "$RADIP" > $TMP_FILE

    if [ -z "$RADIP" ]; then
        undo_firewall
        undo_routing
        update_stop
        log "Going offline"
        return
    fi

    local PREFIX_LAN="$(echo "$RADIP" | sed -nr 's~^(\w+:\w+:\w+:\w+):.*(/.*)~\1::\2~p')"
    [ "$(echo "$PREFIX_LAN" | sed -r 's~.*(.).::/..~\1~')" -le 7 2>/dev/null ] && NEWPRE='8' || NEWPRE='0'
    local PREFIX_DEL="$(echo "$PREFIX_LAN" | sed "s~..::/..$~${NEWPRE}0~")::/57"
    local PREFIX_ALL="$(echo "$PREFIX_LAN" | sed "s~..::/..$~00~")::/56"

    log "> IP6: $RADIP"
    log "> LAN: $PREFIX_LAN"
    log "> DEL: $PREFIX_DEL"
    log "> ALL: $PREFIX_ALL"

    if  ! xtables-multi ip6tables-save | grep -qF "$PREFIX_DEL"; then
        undo_firewall

        ip6tables -t mangle -A PRE_LAN_SUBNET -s $PREFIX_DEL -i br0 -j ACCEPT
        ip6tables -t mangle -I ROUTE_CTL_LIST -d $PREFIX_DEL -j RETURN
        ip6tables -F FORWARD_PREFIX
        ip6tables -A FORWARD_PREFIX ! -s $PREFIX_ALL -i br0 -j REJECT --reject-with icmp6-dst-unreachable
        ip6tables -D FORWARD_FIREWALL ! -i br+ -j DROP
        ip6tables -A FORWARD_FIREWALL -d $PREFIX_DEL -i gre+ -o br0 -j ACCEPT
        ip6tables -A FORWARD_FIREWALL -d $PREFIX_DEL -i ppp256 -o br0 -j ACCEPT
        ip6tables -A FORWARD_FIREWALL ! -i br+ -j DROP

        echo "ip6tables -t mangle -D PRE_LAN_SUBNET -s $PREFIX_DEL -i br0 -j ACCEPT"    >> /opt/var/lib/dibbler/undo_firewall.sh
        echo "ip6tables -t mangle -D ROUTE_CTL_LIST -d $PREFIX_DEL -j RETURN"           >> /opt/var/lib/dibbler/undo_firewall.sh
        echo "ip6tables -D FORWARD_FIREWALL -d $PREFIX_DEL -i gre+ -o br0 -j ACCEPT"    >> /opt/var/lib/dibbler/undo_firewall.sh
        echo "ip6tables -D FORWARD_FIREWALL -d $PREFIX_DEL -i ppp256 -o br0 -j ACCEPT"  >> /opt/var/lib/dibbler/undo_firewall.sh

        log "Firewall updated"
    fi

    if ! ip -6 route | grep -qF "$PREFIX_DEL"; then
        undo_routing
    fi

    local OLD_POOL="$(grep 'pd-pool ' /opt/etc/dibbler/server.conf 2>/dev/null)"
    sed "s~pd-pool .*~pd-pool $PREFIX_DEL~" -i /opt/etc/dibbler/server.conf
    local NEW_POOL="$(grep 'pd-pool ' /opt/etc/dibbler/server.conf 2>/dev/null)"
    if [ "$OLD_POOL" != "$NEW_POOL" ]; then
        update_stop
        log "Dibbler updated"
    fi

    update_start
}

run() {
    echo "$$" > "$PID_SCRIPT"

    while ! ip -6 addr list dev br0 scope global 2>/dev/null | grep -q ' 2003:'; do
        log "Warmup"
        sleep 10
    done

    while true; do
        log "Running"
        update

        log "Waiting"
        log '#'
        inotifywait -q -e close_write $RADVD_CONF >/dev/null 2>>$LOG_FILE &
        local PID="$!"
        echo "$PID" > "$PID_DAEMON"
        wait "$PID"

        log "Sleeping"
        sleep 3
    done
}

start() {
    log '#'
    log "Starting"
    cd /tmp
    nohup $0 run 0</dev/null >/dev/null 2>&1 &
}

stop() {
    for PID in $(cat "$PID_SCRIPT" "$PID_DAEMON" 2>/dev/null); do
        kill -0 "$PID" >/dev/null 2>&1 && kill -9 "$PID" >/dev/null 2>&1
    done
    rm -f "$PID_SCRIPT" "$PID_DAEMON"
}

case $1 in
    run)
        run
        ;;
    start|load)
        stop
        start
        ;;
    stop)
        stop
        log "Stopped"
        ;;
    force)
        stop
        update_stop
        rm -f /opt/var/lib/dibbler/* 2>/dev/null
        start
        ;;
    log)
        clear
        echo " #  $LOG_FILE"
        tail -fn999 "$LOG_FILE" 2>/dev/null
        ;;
    *)
        echo "Usage: $0 [start|stop|force|log]" 1>&2
        exit 1
        ;;
esac

exit 0
